#!/usr/bin/perl
# nagios: -epn
## $Id: check_apc by janthony Exp / perl by J.Ph. LEMOINE$

## Nagios Plugin for apcupsd APC Smart-UPS
## janthony@negative1.org

## modified by Jean-Philippe LEMOINE (Add status parameter)
## modified for being in perl for purpose of being 
## easily translate.

## Copyright (C) 2004  Joe Anthony
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use warnings;
use Switch;

sub try_load_module($) {
    my $module = shift;

    eval "require $module";
    if ($@) {
     print "DOMAIN UNKNOWN - Can't load module $module, ",
     "perhaps it isn't installed\n$@";
     print "Config::INI::Reader --> sudo apt-get install libconfig-ini-perl\n";
     print "Nagios::Plugin      --> sudo apt-get install libnagios-pluging-perl\n";
     exit 3;
    }

    $module->import;
   
}

BEGIN {
    try_load_module 'Config::INI::Reader';
    try_load_module 'Nagios::Plugin';
}


#use Config::INI::Reader;
#use Nagios::Plugin;
use vars qw/ $VERSION /;
use lib "/usr/lib/perl5/B";

my $VERSION = '1.1';
my $LICENCE = "Ce plugin Nagios est gratuit et libre de droits, et vous pouvez l'utiliser à votre convenance."
  . ' Il est livré avec ABSOLUMENT AUCUNE GARANTIE.';

my $plugin_nagios = Nagios::Plugin->new(
  shortname => 'check apcupsd',
  usage     => 'Usage : %s -a|--action <action> -w|--warning <warning value> -c|--critical <critical value> -l|--lang <language code fr or en> -i|--ini_path <path ini file>',
  version   => $VERSION,
  license   => $LICENCE,
);

# put the 2 letters of the country code
# in 10/28/2012, there is 2 possibilies :
# fr for french
# en for english
# can be changed if needed
my $country_code_def="fr";

# start with a / is absolute path
# with 1st directory if relative path
# can be changed if needed
my $path_to_ini="ini";

# variables declaration
# do not modify
# command parameter
my $APCA_ARGS="status";
# script pamameter
my $ACTION       = '';
my $WARN         = '';
my $CRIT         = '';
my $COUNTRY_CODE = '';
my $INI_PATH     = '';
# name of the perl script
my $PROGNAME     = '';
# path to perl script
my $PATH         = '';
# name of the ini file
my $NAME_INI     = '';
# content of the ini file = traduction
my $hash_ref     = '';
# for testing if APCACCESS is installed
my $test_install = '';
# path & name oh the ini file
my $ini_file     = '';
# full path of APCACCESS file
my $APCA         = '';
# dummy value (result of APCACCESS w/o parameter)
my $dummy        = '';
# position of separator character
my $possep       = '';
# result of action
my $result       = 0;

# define the current path as possible search path
my $PATH_PL = '';
BEGIN {
    $PATH_PL = `dirname $0`;
    $PATH_PL = substr($PATH_PL, 0, -1);
    if (substr($PATH_PL,0,2) eq "./")
        {
        $PATH_PL=substr($PATH_PL,2);
        }
}

use lib "${PATH_PL}";
# sub routines are in sub file
require "check_apcupsd_sub.pl";

$plugin_nagios->add_arg(
spec     => 'action|a=s',
help     => '<load|bcharge|time|status>',    # Help of option
required => 1,                  # required
);

$plugin_nagios->add_arg(
spec     => 'lang|l=s',
help     => 'fr ou en',    # Help of option
required => 0,                  # optional
);

$plugin_nagios->add_arg(
spec     => 'ini_path|i=s',
help     => 'ini path',    # Help of option
required => 0,                  # optional
);
$plugin_nagios->add_arg(
spec     => 'warning|w=s',
help     => 'value warning state',    # Help of option
required => 0,                  # optional
);

$plugin_nagios->add_arg(
spec     => 'critical|c=s',
help     => 'value critical state',    # Help of option
required => 0,                  # optional
);

$plugin_nagios->getopts();

$ACTION       = $plugin_nagios->opts->get('action');
$WARN         = $plugin_nagios->opts->get('warning');
$CRIT         = $plugin_nagios->opts->get('critical');
$COUNTRY_CODE = $plugin_nagios->opts->get('lang');
$INI_PATH     = $plugin_nagios->opts->get('ini_path');

if (defined($INI_PATH))
    {
    $path_to_ini=$INI_PATH;
    }

# check parameter : if action is different of status, warn & crit is mandatory
if (($ACTION ne 'load') &&
    ($ACTION ne 'bcharge') &&
    ($ACTION ne 'time') &&
    ($ACTION ne 'status') &&
    ($ACTION ne 'help'))
    {
    print "Action : <load|bcharge|time|status|help>";
    exit 3;
    }

if (($ACTION ne 'status') && ($ACTION ne 'help'))
    {
    # warn & crit is mandatory if action is nor status nor help
    if (!(defined($WARN)))
        {
        &display_usage("--warning or -w for warning value.");
        exit 3;
        }
    if (!(defined($CRIT)))
        {
        &display_usage("--critical or -c for critical value.");
        exit 3;
        }
    }

# ---- end parameters retrieve

# check if translation file exists
# if not, try to use default one

if (!(defined($COUNTRY_CODE)))
    {
    $COUNTRY_CODE = $country_code_def;
    }

# retrieve name of pl script
$PROGNAME=`basename $0`;
chomp($PROGNAME);


# retrieve the path of the script
$PATH=get_ini_path($path_to_ini);

# name of ini file is the same of pl script exect extension (.pl)
$NAME_INI=substr($PROGNAME, 0, -3);

# ini_file content the path & name of the ini file
$ini_file = "${PATH}${NAME_INI}_${COUNTRY_CODE}.ini";

if (!(-e $ini_file))
    {
    # ini file is not beeing found , try with default name
    $ini_file = "${PATH}${NAME_INI}.ini";
    if (!(-e $ini_file))
        {
        print "fatal error : conf file (${ini_file}) not found";
        exit 3;
        } 
    }

#put translations in memory
$hash_ref = Config::INI::Reader->read_file($ini_file);

# check if apcacess has been installed

$test_install=`whereis apcaccess`;
chomp($test_install);

# delete apcaccess: string
$test_install = trim(substr($test_install,10));


if ($test_install eq '')
    {
    #fatal error : apcaccess is not installed;
    my $error02a=$hash_ref->{error}->{error02a};
    print "APCACCESS ${error02a}";
    exit 3;
    }
else
    {
    # retrieve the 1st path
    # search if there is a space : separtor if many path
    $possep = index($test_install,' ');
    if ($possep != -1)
        {
        $test_install=trim(substr($test_install,0,$possep));
        }
    $APCA = $test_install;
    }


# test if apcaccess is available
if (&test_apaccess ($hash_ref)!=0 )
    {
    exit 3;
    }

switch ($ACTION)  { 
    case "load"
        {
        $result = action_load($hash_ref, $WARN, $CRIT, $APCA, $APCA_ARGS);
	exit $result;
        }
    case "bcharge"
        {
        $result = action_bcharge($hash_ref, $WARN, $CRIT, $APCA, $APCA_ARGS);
        exit $result;
        }
    case "time"
        {
        $result = action_time($hash_ref, $WARN, $CRIT, $APCA, $APCA_ARGS);
        exit $result;
        }
    case "status"
        {
        $result = action_status($hash_ref, $APCA, $APCA_ARGS);
        exit $result;
        }
   else
        {
        dishelp($hash_ref);
        exit 3;
        }
}
exit 3;


